<?php
namespace Ak\Mail\Transport;

use Interop\Container\ContainerInterface;
use Zend\Mail\Transport\TransportInterface;
use Zend\ServiceManager\Exception\ServiceNotCreatedException;
use Zend\ServiceManager\Factory\FactoryInterface;

/**
 * Stvara novu instancu Sendgrid transporta.
 *
 * @package Ak\Mail\Transport
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/mail
 */
final class SendgridFactory implements FactoryInterface
{
    /**
     * Create an object
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return TransportInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $config = $container->get('config')['sendgrid'] ?? [];

        if (!isset($config['api_key']) || empty($config['api_key'])) {
            throw new ServiceNotCreatedException('Api ključ za sendgrid nije postavljen.');
        }

        return new Sendgrid($config['api_key']);
    }
}