<?php
namespace AkTest\Mail\Transport;

use Ak\Mail\Transport\TransportPluginManager;
use Ak\Test\AbstractTestCase;
use AkTest\Mail\TestConfigProvider;
use Zend\Mail\Transport\File;
use Zend\Mail\Transport\InMemory;
use Zend\Mail\Transport\Sendmail;
use Zend\Mail\Transport\Smtp;

/**
 * Class TransportPluginManagerTest
 *
 * @package AkTest\Mail\Transport
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/mail
 */
final class TransportPluginManagerTest extends AbstractTestCase
{
    public $configProviders = [
        TestConfigProvider::class,
    ];

    protected function getTransportPluginManager(): TransportPluginManager
    {
        $container = $this->getContainer();
        return $container->get('mail.transports');
    }

    public function testIsRegistered()
    {
        $container = $this->getContainer();

        $this->assertContainerHasService($container, [
            'mail.transports',
            TransportPluginManager::class,
        ], TransportPluginManager::class, true);
    }

    public function testCanCreateSendmailTransport()
    {
        $transports = $this->getTransportPluginManager();
        $this->assertContainerHasService($transports, 'sendmail', Sendmail::class);
    }

    public function testCanCreateSmtpTransport()
    {
        $tranports = $this->getTransportPluginManager();
        $this->assertTrue($tranports->has('smtp'));

        /** @var Smtp $smtp */
        $smtp = $tranports->get('smtp', [
            'name' => 'test-transport',
            'host' => 'smtp.example.com',
            'port' => 25,
            'connection_class' => 'crammd5',
            'connection_config' => [
                'username' => 'smtpuser',
                'password' => 'smtppassword',
            ],
        ]);

        $this->assertInstanceOf(Smtp::class, $smtp);

        $options = $smtp->getOptions();
        $this->assertEquals('test-transport', $options->getName());
        $this->assertEquals('smtp.example.com', $options->getHost());
        $this->assertEquals(25, $options->getPort());
    }

    public function testCanCreateFileTransport()
    {
        $transports = $this->getTransportPluginManager();
        $this->assertTrue($transports->has('file'));

        $callback = function() {};

        /** @var File $file */
        $file = $transports->get('file', [
            'path' => __DIR__,
            'callback' => $callback,
        ]);

        $this->assertInstanceOf(File::class, $file);

        $options = $file->getOptions();
        $this->assertEquals(__DIR__, $options->getPath());
        $this->assertSame($callback, $options->getCallback());
    }

    public function testCanCreateInMemoryTransport()
    {
        $transports = $this->getTransportPluginManager();
        $this->assertTrue($transports->has('inmemory'));

        /** @var InMemory $file */
        $file = $transports->get('inmemory');

        $this->assertInstanceOf(InMemory::class, $file);
    }
}